package epazote

import (
	"errors"
)

// Func represents functions that can be retried.
type Func func(attempt int) (retry bool, err error)

// Try Do keeps trying the function until the second argument
// returns false, or no error is returned.
func Try(fn Func) error {
	var err error
	var cont bool
	attempt := 1
	for {
		cont, err = fn(attempt)
		if !cont || err == nil {
			break
		}
		attempt++
		if attempt > 1<<16 {
			return errors.New("Exceeded retry limit")
		}
	}
	return err
}
